[{"locale":"es"},{"key":"ns","names":["ns"],"mappings":{"default":{"default":"nanosegundos"}},"category":"time"},{"key":"µs","names":["µs"],"mappings":{"default":{"default":"microsegundos"}},"category":"time"},{"key":"ms","names":["ms"],"mappings":{"default":{"default":"milisegundos"}},"category":"time"},{"key":"s","names":["s"],"mappings":{"default":{"default":"segundos"}},"category":"time"},{"key":"min","names":["min"],"mappings":{"default":{"default":"minutos"}},"category":"time"},{"key":"h","names":["h","hr"],"mappings":{"default":{"default":"hora"}},"category":"time"}]
